@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2021, The PurpleI2P Project
+* Copyright (c) 2021-2023, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -31,15 +31,16 @@ namespace russian // language namespace
 
 	static std::map<std::string, std::string> strings
 	{
-		{"KiB", "КиБ"},
-		{"MiB", "МиБ"},
-		{"GiB", "ГиБ"},
+		{"%.2f KiB", "%.2f КиБ"},
+		{"%.2f MiB", "%.2f МиБ"},
+		{"%.2f GiB", "%.2f ГиБ"},
 		{"building", "строится"},
 		{"failed", "неудачный"},
 		{"expiring", "истекает"},
 		{"established", "работает"},
 		{"unknown", "неизвестно"},
 		{"exploratory", "исследовательский"},
+		{"Purple I2P Webconsole", "Веб-консоль Purple I2P"},
 		{"<b>i2pd</b> webconsole", "Веб-консоль <b>i2pd</b>"},
 		{"Main page", "Главная"},
 		{"Router commands", "Команды роутера"},
@@ -57,18 +58,20 @@ namespace russian // language namespace
 		{"Unknown", "Неизвестно"},
 		{"Proxy", "Прокси"},
 		{"Mesh", "MESH-сеть"},
-		{"Error", "Ошибка"},
 		{"Clock skew", "Не точное время"},
 		{"Offline", "Оффлайн"},
 		{"Symmetric NAT", "Симметричный NAT"},
+		{"Full cone NAT", "Full cone NAT"},
+		{"No Descriptors", "Нет дескрипторов"},
 		{"Uptime", "В сети"},
 		{"Network status", "Сетевой статус"},
 		{"Network status v6", "Сетевой статус v6"},
 		{"Stopping in", "Остановка через"},
 		{"Family", "Семейство"},
 		{"Tunnel creation success rate", "Успешно построенных туннелей"},
+		{"Total tunnel creation success rate", "Общий процент успешно построенных туннелей"},
 		{"Received", "Получено"},
-		{"KiB/s", "КиБ/с"},
+		{"%.2f KiB/s", "%.2f КиБ/с"},
 		{"Sent", "Отправлено"},
 		{"Transit", "Транзит"},
 		{"Data path", "Путь к данным"},
@@ -93,8 +96,9 @@ namespace russian // language namespace
 		{"Address", "Адрес"},
 		{"Type", "Тип"},
 		{"EncType", "ТипШифр"},
+		{"Expire LeaseSet", "Просрочить Лизсет"},
 		{"Inbound tunnels", "Входящие туннели"},
-		{"ms", "мс"},
+		{"%dms", "%dмс"},
 		{"Outbound tunnels", "Исходящие туннели"},
 		{"Tags", "Теги"},
 		{"Incoming", "Входящие"},
@@ -107,6 +111,7 @@ namespace russian // language namespace
 		{"Local Destination", "Локальное назначение"},
 		{"Streams", "Стримы"},
 		{"Close stream", "Закрыть стрим"},
+		{"Such destination is not found", "Такая точка назначения не найдена"},
 		{"I2CP session not found", "I2CP сессия не найдена"},
 		{"I2CP is not enabled", "I2CP не включен"},
 		{"Invalid", "Некорректный"},
@@ -116,9 +121,10 @@ namespace russian // language namespace
 		{"Gateway", "Шлюз"},
 		{"TunnelID", "ID туннеля"},
 		{"EndDate", "Заканчивается"},
-		{"not floodfill", "не флудфил"},
+		{"floodfill mode is disabled", "режим флудфила отключен"},
 		{"Queue size", "Размер очереди"},
 		{"Run peer test", "Запустить тестирование"},
+		{"Reload tunnels configuration", "Перезагрузить конфигурацию туннелей"},
 		{"Decline transit tunnels", "Отклонять транзитные туннели"},
 		{"Accept transit tunnels", "Принимать транзитные туннели"},
 		{"Cancel graceful shutdown", "Отменить плавную остановку"},
@@ -146,8 +152,10 @@ namespace russian // language namespace
 		{"Destination not found", "Точка назначения не найдена"},
 		{"StreamID can't be null", "StreamID не может быть пустым"},
 		{"Return to destination page", "Вернуться на страницу точки назначения"},
-		{"You will be redirected in 5 seconds", "Вы будете переадресованы через 5 секунд"},
-		{"Transit tunnels count must not exceed 65535", "Число транзитных туннелей не должно превышать 65535"},
+		{"You will be redirected in %d seconds", "Вы будете переадресованы через %d секунд"},
+		{"LeaseSet expiration time updated", "Время действия LeaseSet обновлено"},
+		{"LeaseSet is not found or already expired", "Лизсет не найден или время действия уже истекло"},
+		{"Transit tunnels count must not exceed %d", "Число транзитных туннелей не должно превышать %d"},
 		{"Back to commands list", "Вернуться к списку команд"},
 		{"Register at reg.i2p", "Зарегистрировать на reg.i2p"},
 		{"Description", "Описание"},
@@ -155,7 +163,6 @@ namespace russian // language namespace
 		{"Submit", "Отправить"},
 		{"Domain can't end with .b32.i2p", "Домен не может заканчиваться на .b32.i2p"},
 		{"Domain must end with .i2p", "Домен должен заканчиваться на .i2p"},
-		{"Such destination is not found", "Такая точка назначения не найдена"},
 		{"Unknown command", "Неизвестная команда"},
 		{"Command accepted", "Команда принята"},
 		{"Proxy error", "Ошибка прокси"},
@@ -165,32 +172,33 @@ namespace russian // language namespace
 		{"You may try to find this host on jump services below", "Вы можете попробовать найти узел через джамп сервисы ниже"},
 		{"Invalid request", "Некорректный запрос"},
 		{"Proxy unable to parse your request", "Прокси не может разобрать ваш запрос"},
-		{"addresshelper is not supported", "addresshelper не поддерживается"},
-		{"Host", "Узел"},
-		{"added to router's addressbook from helper", "добавлен в адресную книгу роутера через хелпер"},
-		{"Click here to proceed:", "Нажмите здесь, чтобы продолжить:"},
-		{"Continue", "Продолжить"},
-		{"Addresshelper found", "Найден addresshelper"},
-		{"already in router's addressbook", "уже в адресной книге роутера"},
-		{"Click here to update record:", "Нажмите здесь, чтобы обновить запись:"},
-		{"invalid request uri", "некорректный URI запроса"},
+		{"Addresshelper is not supported", "Addresshelper не поддерживается"},
+		{"Host %s is <font color=red>already in router's addressbook</font>. <b>Be careful: source of this URL may be harmful!</b> Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "Узел %s <font color=red>уже в адресной книге роутера</font>. <b>Будьте осторожны: источник данной ссылки может быть вредоносным!</b> Нажмите здесь, чтобы обновить запись: <a href=\"%s%s%s&update=true\">Продолжить</a>."},
+		{"Addresshelper forced update rejected", "Принудительное обновление через Addresshelper отклонено"},
+		{"To add host <b>%s</b> in router's addressbook, click here: <a href=\"%s%s%s\">Continue</a>.", "Чтобы добавить узел <b>%s</b> в адресную книгу роутера, нажмите здесь: <a href=\"%s%s%s\">Продолжить</a>."},
+		{"Addresshelper request", "Запрос добавления Addresshelper"},
+		{"Host %s added to router's addressbook from helper. Click here to proceed: <a href=\"%s\">Continue</a>.", "Узел %s добавлен в адресную книгу роутера через хелпер. Нажмите здесь, чтобы продолжить: <a href=\"%s\">Продолжить</a>."},
+		{"Addresshelper adding", "Добавление Addresshelper"},
+		{"Host %s is <font color=red>already in router's addressbook</font>. Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "Узел %s <font color=red>уже в адресной книге роутера</font>. Нажмите здесь, чтобы обновить запись: <a href=\"%s%s%s&update=true\">Продолжить</a>."},
+		{"Addresshelper update", "Обновление записи через Addresshelper"},
+		{"Invalid request URI", "Некорректный URI запроса"},
 		{"Can't detect destination host from request", "Не удалось определить адрес назначения из запроса"},
 		{"Outproxy failure", "Ошибка внешнего прокси"},
-		{"bad outproxy settings", "некорректные настройки внешнего прокси"},
-		{"not inside I2P network, but outproxy is not enabled", "не в I2P сети, но внешний прокси не включен"},
-		{"unknown outproxy url", "неизвестный URL внешнего прокси"},
-		{"cannot resolve upstream proxy", "не удается определить вышестоящий прокси"},
-		{"hostname too long", "имя хоста слишком длинное"},
-		{"cannot connect to upstream socks proxy", "не удается подключиться к вышестоящему SOCKS прокси"},
-		{"Cannot negotiate with socks proxy", "Не удается договориться с вышестоящим SOCKS прокси"},
+		{"Bad outproxy settings", "Некорректные настройки внешнего прокси"},
+		{"Host %s is not inside I2P network, but outproxy is not enabled", "Узел %s не в I2P сети, но внешний прокси не включен"},
+		{"Unknown outproxy URL", "Неизвестный URL внешнего прокси"},
+		{"Cannot resolve upstream proxy", "Не удается определить вышестоящий прокси"},
+		{"Hostname is too long", "Имя хоста слишком длинное"},
+		{"Cannot connect to upstream SOCKS proxy", "Не удалось подключиться к вышестоящему SOCKS прокси серверу"},
+		{"Cannot negotiate with SOCKS proxy", "Не удается договориться с вышестоящим SOCKS прокси"},
 		{"CONNECT error", "Ошибка CONNECT запроса"},
-		{"Failed to Connect", "Не удалось подключиться"},
-		{"socks proxy error", "ошибка SOCKS прокси"},
-		{"failed to send request to upstream", "не удалось отправить запрос вышестоящему прокси"},
-		{"No Reply From socks proxy", "Нет ответа от SOCKS прокси сервера"},
-		{"cannot connect", "не удалось подключиться"},
-		{"http out proxy not implemented", "поддержка внешнего HTTP прокси сервера не реализована"},
-		{"cannot connect to upstream http proxy", "не удалось подключиться к вышестоящему HTTP прокси серверу"},
+		{"Failed to connect", "Не удалось соединиться"},
+		{"SOCKS proxy error", "Ошибка SOCKS прокси"},
+		{"Failed to send request to upstream", "Не удалось отправить запрос вышестоящему прокси серверу"},
+		{"No reply from SOCKS proxy", "Нет ответа от SOCKS прокси сервера"},
+		{"Cannot connect", "Не удалось подключиться"},
+		{"HTTP out proxy not implemented", "Поддержка внешнего HTTP прокси сервера не реализована"},
+		{"Cannot connect to upstream HTTP proxy", "Не удалось подключиться к вышестоящему HTTP прокси серверу"},
 		{"Host is down", "Узел недоступен"},
 		{"Can't create connection to requested host, it may be down. Please try again later.", "Не удалось установить соединение к запрошенному узлу, возможно он не в сети. Попробуйте повторить запрос позже."},
 		{"", ""},
@@ -198,10 +206,10 @@ namespace russian // language namespace
 
 	static std::map<std::string, std::vector<std::string>> plurals
 	{
-		{"days",    {"день", "дня", "дней"}},
-		{"hours",   {"час", "часа", "часов"}},
-		{"minutes", {"минуту", "минуты", "минут"}},
-		{"seconds", {"секунду", "секунды", "секунд"}},
+		{"%d days",    {"%d день", "%d дня", "%d дней"}},
+		{"%d hours",   {"%d час", "%d часа", "%d часов"}},
+		{"%d minutes", {"%d минуту", "%d минуты", "%d минут"}},
+		{"%d seconds", {"%d секунду", "%d секунды", "%d секунд"}},
 		{"", {"", "", ""}},
 	};
 
