@@ -1,5 +1,5 @@
 /*
-* Copyright (c) 2021-2022, The PurpleI2P Project
+* Copyright (c) 2021-2023, The PurpleI2P Project
 *
 * This file is part of Purple i2pd project and licensed under BSD3
 *
@@ -31,15 +31,16 @@ namespace uzbek // language namespace
 
 	static std::map<std::string, std::string> strings
 	{
-		{"KiB", "KiB"},
-		{"MiB", "MiB"},
-		{"GiB", "GiB"},
+		{"%.2f KiB", "%.2f KiB"},
+		{"%.2f MiB", "%.2f MiB"},
+		{"%.2f GiB", "%.2f GiB"},
 		{"building", "yaratilmoqda"},
 		{"failed", "muvaffaqiyatsiz"},
 		{"expiring", "muddati tugaydi"},
 		{"established", "aloqa o'rnatildi"},
 		{"unknown", "noma'lum"},
 		{"exploratory", "tadqiqiy"},
+		{"Purple I2P Webconsole", "Veb-konsoli Purple I2P"},
 		{"<b>i2pd</b> webconsole", "<b>i2pd</b> veb-konsoli"},
 		{"Main page", "Asosiy sahifa"},
 		{"Router commands", "Router buyruqlari"},
@@ -57,18 +58,20 @@ namespace uzbek // language namespace
 		{"Unknown", "Notanish"},
 		{"Proxy", "Proksi"},
 		{"Mesh", "Mesh To'r"},
-		{"Error", "Xato"},
 		{"Clock skew", "Aniq vaqt emas"},
 		{"Offline", "Oflayn"},
 		{"Symmetric NAT", "Simmetrik NAT"},
+		{"Full cone NAT", "Full cone NAT"},
+		{"No Descriptors", "Deskriptorlar yo'q"},
 		{"Uptime", "Ish vaqti"},
 		{"Network status", "Tarmoq holati"},
 		{"Network status v6", "Tarmoq holati v6"},
 		{"Stopping in", "Ichida to'xtatish"},
 		{"Family", "Oila"},
 		{"Tunnel creation success rate", "Tunnel yaratish muvaffaqiyat darajasi"},
+		{"Total tunnel creation success rate", "Tunnel yaratishning umumiy muvaffaqiyat darajasi"},
 		{"Received", "Qabul qilindi"},
-		{"KiB/s", "KiB/s"},
+		{"%.2f KiB/s", "%.2f KiB/s"},
 		{"Sent", "Yuborilgan"},
 		{"Transit", "Tranzit"},
 		{"Data path", "Ma'lumotlar joylanishi"},
@@ -93,8 +96,9 @@ namespace uzbek // language namespace
 		{"Address", "Manzil"},
 		{"Type", "Turi"},
 		{"EncType", "ShifrlashTuri"},
+		{"Expire LeaseSet", "LeaseSet muddati tugaydi"},
 		{"Inbound tunnels", "Kirish tunnellari"},
-		{"ms", "ms"},
+		{"%dms", "%dms"},
 		{"Outbound tunnels", "Chiquvchi tunnellar"},
 		{"Tags", "Teglar"},
 		{"Incoming", "Kiruvchi"},
@@ -107,6 +111,7 @@ namespace uzbek // language namespace
 		{"Local Destination", "Mahalliy joylanish"},
 		{"Streams", "Strim"},
 		{"Close stream", "Strimni o'chirish"},
+		{"Such destination is not found", "Bunday yo'nalish topilmadi"},
 		{"I2CP session not found", "I2CP sessiyasi topilmadi"},
 		{"I2CP is not enabled", "I2CP yoqilmagan"},
 		{"Invalid", "Noto'g'ri"},
@@ -116,9 +121,10 @@ namespace uzbek // language namespace
 		{"Gateway", "Kirish yo'li"},
 		{"TunnelID", "TunnelID"},
 		{"EndDate", "Tugash Sanasi"},
-		{"not floodfill", "floodfill emas"},
+		{"floodfill mode is disabled", "floodfill rejimi o'chirilgan"},
 		{"Queue size", "Navbat hajmi"},
 		{"Run peer test", "Sinovni boshlang"},
+		{"Reload tunnels configuration", "Tunnel konfiguratsiyasini qayta yuklash"},
 		{"Decline transit tunnels", "Tranzit tunnellarini rad etish"},
 		{"Accept transit tunnels", "Tranzit tunnellarni qabul qilish"},
 		{"Cancel graceful shutdown", "Yumshoq to'xtashni bekor qilish"},
@@ -146,8 +152,10 @@ namespace uzbek // language namespace
 		{"Destination not found", "Yo'nalish topilmadi"},
 		{"StreamID can't be null", "StreamID bo'sh bo'lishi mumkin emas"},
 		{"Return to destination page", "Manzilgoh sahifasiga qaytish"},
-		{"You will be redirected in 5 seconds", "Siz 5 soniya ichida qayta yo'naltirilasiz"},
-		{"Transit tunnels count must not exceed 65535", "Tranzit tunnellar soni 65535 dan oshmasligi kerak"},
+		{"You will be redirected in %d seconds", "Siz %d soniyadan so‘ng boshqa yo‘nalishga yo‘naltirilasiz"},
+		{"LeaseSet expiration time updated", "LeaseSet amal qilish muddati yangilandi"},
+		{"LeaseSet is not found or already expired", "LeaseSet topilmadi yoki muddati tugagan"},
+		{"Transit tunnels count must not exceed %d", "Tranzit tunnellar soni %d dan oshmasligi kerak"},
 		{"Back to commands list", "Buyruqlar ro'yxatiga qaytish"},
 		{"Register at reg.i2p", "Reg.i2p-da ro'yxatdan o'ting"},
 		{"Description", "Tavsif"},
@@ -155,7 +163,6 @@ namespace uzbek // language namespace
 		{"Submit", "Yuborish"},
 		{"Domain can't end with .b32.i2p", "Domen .b32.i2p bilan tugashi mumkin emas"},
 		{"Domain must end with .i2p", "Domen .i2p bilan tugashi kerak"},
-		{"Such destination is not found", "Bunday yo'nalish topilmadi"},
 		{"Unknown command", "Noma'lum buyruq"},
 		{"Command accepted", "Buyruq qabul qilindi"},
 		{"Proxy error", "Proksi xatosi"},
@@ -165,32 +172,33 @@ namespace uzbek // language namespace
 		{"You may try to find this host on jump services below", "Siz xost quyida o'tish xizmatlari orqali topishga harakat qilishingiz mumkin"},
 		{"Invalid request", "Noto‘g‘ri so‘rov"},
 		{"Proxy unable to parse your request", "Proksi sizning so'rovingizni aniqlab ololmayapti"},
-		{"addresshelper is not supported", "addresshelper qo'llab -quvvatlanmaydi"},
-		{"Host", "Xost"},
-		{"added to router's addressbook from helper", "'helper'dan routerning 'addressbook'ga qo'shildi"},
-		{"Click here to proceed:", "Davom etish uchun shu yerni bosing:"},
-		{"Continue", "Davom etish"},
-		{"Addresshelper found", "Addresshelper topildi"},
-		{"already in router's addressbook", "allaqachon 'addressbook'da yozilgan"},
-		{"Click here to update record:", "Yozuvni yangilash uchun shu yerni bosing:"},
-		{"invalid request uri", "noto'g'ri URI so'rovi"},
+		{"Addresshelper is not supported", "Addresshelper qo'llab-quvvatlanmaydi"},
+		{"Host %s is <font color=red>already in router's addressbook</font>. <b>Be careful: source of this URL may be harmful!</b> Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "%s xosti <font color=red>allaqachon routerning manzillar kitobida</font>. <b>Ehtiyot bo'ling: bu URL manbasi zararli bo'lishi mumkin!</b> Yozuvni yangilash uchun bu yerni bosing: <a href=\"%s%s%s&update=true\">Davom etish</a>."},
+		{"Addresshelper forced update rejected", "Addresshelperni majburiy yangilash rad etildi"},
+		{"To add host <b>%s</b> in router's addressbook, click here: <a href=\"%s%s%s\">Continue</a>.", "Routerning manzillar kitobiga <b>%s</b> xostini qo'shish uchun bu yerni bosing: <a href=\"%s%s%s\">Davom etish</a>."},
+		{"Addresshelper request", "Addresshelper so'rovi"},
+		{"Host %s added to router's addressbook from helper. Click here to proceed: <a href=\"%s\">Continue</a>.", "Yordamchidan router manzillar kitobiga %s xost qo‘shildi. Davom etish uchun bu yerga bosing: <a href=\"%s\">Davom etish</a>."},
+		{"Addresshelper adding", "Addresshelperni qo'shish"},
+		{"Host %s is <font color=red>already in router's addressbook</font>. Click here to update record: <a href=\"%s%s%s&update=true\">Continue</a>.", "%s xosti <font color=red>allaqachon routerning manzillar kitobida</font>. Yozuvni yangilash uchun shu yerni bosing: <a href=\"%s%s%s&update=true\">Davom etish</a>."},
+		{"Addresshelper update", "Addresshelperni yangilash"},
+		{"Invalid request URI", "Noto'g'ri URI so'rovi"},
 		{"Can't detect destination host from request", "So‘rov orqali manzil xostini aniqlab bo'lmayapti"},
 		{"Outproxy failure", "Tashqi proksi muvaffaqiyatsizligi"},
-		{"bad outproxy settings", "noto'g'ri tashqi proksi-server sozlamalari"},
-		{"not inside I2P network, but outproxy is not enabled", "I2P tarmog'ida emas, lekin tashqi proksi yoqilmagan"},
-		{"unknown outproxy url", "noma'lum outproxy url"},
-		{"cannot resolve upstream proxy", "yuqoridagi 'proxy-server'ni aniqlab olib bolmayapti"},
-		{"hostname too long", "xost nomi juda uzun"},
-		{"cannot connect to upstream socks proxy", "yuqori 'socks proxy'ga ulanib bo'lmayapti"},
-		{"Cannot negotiate with socks proxy", "'Socks proxy' bilan muzokara olib bo'lmaydi"},
+		{"Bad outproxy settings", "Noto'g'ri tashqi proksi-server sozlamalari"},
+		{"Host %s is not inside I2P network, but outproxy is not enabled", "Xost %s I2P tarmog'ida emas, lekin tashqi proksi yoqilmagan"},
+		{"Unknown outproxy URL", "Noma'lum outproxy URL"},
+		{"Cannot resolve upstream proxy", "Yuqoridagi 'proxy-server'ni aniqlab olib bolmayapti"},
+		{"Hostname is too long", "Xost nomi juda uzun"},
+		{"Cannot connect to upstream SOCKS proxy", "Yuqori 'SOCKS proxy'ga ulanib bo'lmayapti"},
+		{"Cannot negotiate with SOCKS proxy", "'SOCKS proxy' bilan muzokara olib bo'lmaydi"},
 		{"CONNECT error", "CONNECT xatosi"},
-		{"Failed to Connect", "Ulanib bo'lmayapti"},
-		{"socks proxy error", "'socks proxy' xatosi"},
-		{"failed to send request to upstream", "yuqori http proksi-serveriga so'rovni uborib bo'lmadi"},
-		{"No Reply From socks proxy", "'Socks proxy'dan javob yo'q"},
-		{"cannot connect", "ulanib bo'lmaydi"},
-		{"http out proxy not implemented", "tashqi HTTP proksi-serverni qo'llab-quvvatlash amalga oshirilmagan"},
-		{"cannot connect to upstream http proxy", "yuqori http 'proxy-server'iga ulanib bo'lmayapti"},
+		{"Failed to connect", "Ulanib bo'lmayapti"},
+		{"SOCKS proxy error", "'SOCKS proxy' xatosi"},
+		{"Failed to send request to upstream", "Yuqori proksi-serveriga so'rovni uborib bo'lmadi"},
+		{"No reply from SOCKS proxy", "'SOCKS proxy'dan javob yo'q"},
+		{"Cannot connect", "Ulanib bo'lmaydi"},
+		{"HTTP out proxy not implemented", "Tashqi HTTP proksi-serverni qo'llab-quvvatlash amalga oshirilmagan"},
+		{"Cannot connect to upstream HTTP proxy", "Yuqori 'HTTP proxy'ga ulanib bo'lmayapti"},
 		{"Host is down", "Xost ishlamayapti"},
 		{"Can't create connection to requested host, it may be down. Please try again later.", "Talab qilingan xost bilan aloqa o'rnatilmadi, u ishlamay qolishi mumkin. Iltimos keyinroq qayta urinib ko'ring."},
 		{"", ""},
@@ -198,10 +206,10 @@ namespace uzbek // language namespace
 
 	static std::map<std::string, std::vector<std::string>> plurals
 	{
-		{"days", {"kun", "kun"}},
-		{"hours", {"soat", "soat"}},
-		{"minutes", {"daqiqa", "daqiqa"}},
-		{"seconds", {"soniya", "soniya"}},
+		{"%d days", {"%d kun", "%d kun"}},
+		{"%d hours", {"%d soat", "%d soat"}},
+		{"%d minutes", {"%d daqiqa", "%d daqiqa"}},
+		{"%d seconds", {"%d soniya", "%d soniya"}},
 		{"", {"", ""}},
 	};
 
